@@ -1,36 +1,61 @@
-//! **Maintainer's Note:** binrw is in a rather early state. Lots of work is being
-//! done in order to make a great library before holding it to too many stability guarantees.
-//! Expect a bit of churn, if you're looking for a stable alternative, check out [`binread`].
-//! If you're interested in getting involved, join us on [discord] or [matrix]! (whichever you
-//! prefer, they're bridged together)
+//! **Maintainer's Note:** binrw is currently considered pre-release. Most features are in-place
+//! but documentation may need work and there may be small breaking changes before release. Need
+//! help or want to contribute? Join us on [discord] or [matrix]! (whichever you prefer, they're
+//! bridged together)
+//!
+//! ---
+//!
+//! |**Quick links**|[`#[br]`](attribute::read)|[`#[bw]`](attribute::write)|[`BinReaderExt`]|[discord]|[matrix]|
+//! |-|-|-|-|-|-|
 //!
 //! ---
 //!
-//! [`binread`]: https://docs.rs/binread
 //! [discord]: https://discord.gg/ABy4Qh549j
 //! [matrix]: https://matrix.to/#/#binrw:matrix.org
 //!
 //! binrw helps you write maintainable & easy-to-read declarative binary data
-//! parsers using ✨macro magic✨.
+//! readers and writers using ✨macro magic✨.
 //!
-//! Adding `#[derive(BinRead)]` to any struct or enum generates a parser that
-//! can read that type from raw data:
+//! Adding [`#[binrw]`](macro@binrw) (or `#[derive(BinRead, BinWrite)]`) to a struct or enum
+//! generates a parser that can read that type from raw data and a writer that
+//! can write it back to bytes:
 //!
 //! ```
-//! # use binrw::{BinRead, io::Cursor};
+//! use binrw::binrw; // #[binrw] attribute
+//! use binrw::{BinReaderExt, BinWrite, io::Cursor}; // reading/writing utilities
+//!
+//! #[binrw]
 //! # #[derive(Debug, PartialEq)]
-//! #[derive(BinRead)]
 //! #[br(little)]
 //! struct Point(i16, i16);
 //!
-//! let point = Point::read(&mut Cursor::new(b"\x80\x02\xe0\x01")).unwrap();
+//! // Read a point from bytes
+//! let point: Point = Cursor::new(b"\x80\x02\xe0\x01").read_le().unwrap();
 //! assert_eq!(point, Point(640, 480));
 //!
+//! // Write the point back to bytes
+//! let mut writer = Cursor::new(Vec::new());
+//! point.write_to(&mut writer).unwrap();
+//! assert_eq!(&writer.into_inner()[..], b"\x80\x02\xe0\x01");
+//! ```
+//!
+//! These types are composable, allowing you to use [`BinRead`]/[`BinWrite`] types within
+//! others without any special logic:
+//!
+//! ```
+//! # use binrw::{binrw, BinRead, BinWrite, io::Cursor};
+//! # #[binrw]
+//! # #[derive(Debug, PartialEq)]
+//! # #[br(little)]
+//! # struct Point(i16, i16);
+//! #
 //! # #[derive(Debug, PartialEq)]
 //! #[derive(BinRead)]
 //! #[br(big, magic = b"SHAP")]
 //! enum Shape {
-//!     #[br(magic(0u8))] Rect { left: i16, top: i16, right: i16, bottom: i16 },
+//!     #[br(magic(0u8))] Rect {
+//!         left: i16, top: i16, right: i16, bottom: i16
+//!     },
 //!     #[br(magic(1u8))] Oval { origin: Point, rx: u8, ry: u8 }
 //! }
 //!
@@ -132,11 +157,14 @@
 #[allow(unused_extern_crates)]
 extern crate self as binrw;
 
+#[doc(hidden)]
 #[cfg(feature = "std")]
-use std as alloc;
+pub use std as alloc;
 
+#[doc(hidden)]
 #[cfg(not(feature = "std"))]
-extern crate alloc;
+pub extern crate alloc;
+
 #[cfg(all(doc, not(feature = "std")))]
 use alloc::vec::Vec;
 
@@ -148,10 +176,11 @@ pub mod attribute;
 pub mod endian;
 pub mod error;
 pub mod file_ptr;
+#[doc(hidden)]
+pub mod has_magic;
 pub mod helpers;
 pub mod io;
-#[doc(hidden)]
-pub mod options;
+
 #[doc(hidden)]
 pub mod pos_value;
 pub mod punctuated;
@@ -163,8 +192,8 @@ pub use {
     endian::Endian,
     error::Error,
     file_ptr::{FilePtr, FilePtr128, FilePtr16, FilePtr32, FilePtr64, FilePtr8},
+    has_magic::HasMagic,
     helpers::{count, until, until_eof, until_exclusive},
-    options::ReadOptions,
     pos_value::PosValue,
     strings::{NullString, NullWideString},
 };
@@ -174,19 +203,30 @@ pub use binrw_derive::BinRead;
 
 /// The attribute version of the derive macro for [`BinRead`]. Use this instead
 /// of `#[derive(BinRead)]` to enable [temporary variables](attribute#temp).
-pub use binrw_derive::derive_binread;
+pub use binrw_derive::binread;
+
+/// The derive macro for [`BinWrite`].
+pub use binrw_derive::BinWrite;
+
+/// The attribute version of the derive macro for [`BinRead`]. Use this instead
+/// of `#[derive(BinRead)]` to enable [temporary variables](attribute#temp).
+pub use binrw_derive::binwrite;
+
+/// The attribute version of the derive macro for both [`BinRead`] and [`BinWrite`]. Use
+/// instead of `#[derive(BinRead, BinWrite)]` to enable [temporary variables](attribute#temp).
+pub use binrw_derive::binrw;
 
 pub use binrw_derive::BinrwNamedArgs;
 
 /// A specialized [`Result`] type for BinRead operations.
 pub type BinResult<T> = core::result::Result<T, Error>;
 
-mod binread_impls;
-pub use binread_impls::*;
-
 mod binread;
 pub use binread::*;
 
+mod binwrite;
+pub use binwrite::*;
+
 mod builder_types;
 pub use builder_types::*;
 
@@ -204,4 +244,6 @@ pub mod prelude {
     pub use crate::BinRead;
     pub use crate::BinReaderExt;
     pub use crate::BinResult;
+    pub use crate::BinWrite;
+    pub use crate::BinWriterExt;
 }
